<?php
/**
 * $Id: OOoTemplate.php 123 2019-09-03 15:03:36Z munix9 $
 * 
 * This class does the OO template handling
 * 
 */

require_once $config->classPath . '/modules/common.php';

/**
 * 
 * 
 *   @package    modules
 *   @version    2002/11/08
 *   @access     public
 *   @author     Wolfram Kriesing <wolfram@kriesing.de>
 */
class modules_OOoTemplate extends modules_common
{

    var $table = TABLE_OOOTEMPLATE;

    /**
     *
     *   @version    2002/11/08
     */
    function __construct()
    {
        parent::__construct();
        $this->preset();
    }

    /**
     *   init the table settings, by default we dont want the blob data
     *
     *   @version    2002/11/08
     *   @access     public
     *   @author     Wolfram Kriesing <wolfram@kriesing.de>
     */
    function preset()
    {
        $this->reset();
        // dont get all the blob data here, by default. if u need them select them explicitly
        $this->setDontSelect('data');
    }

    /**
     *   add the timestamp, so its saved in the db too
     *
     *   @version    2002/11/08
     *   @access     public
     *   @author     Wolfram Kriesing <wolfram@kriesing.de>
     *   @param      array   all the data
     *   @return     mixed   either false, or the id of the inserted item
     */
    function add($data)
    {
        $data['timestamp'] = time();
        return parent::add($data);
    }

    /**
     *   this gets the filename for the given id and puts the data from the db in the tmp-dir
     *   if the template for the id is not in the filesystem yet it gets the blob data
     *   and saves them as a template file in the tmp-dir
     *
     *   @version    2002/11/08
     *   @access     public
     *   @author     Wolfram Kriesing <wolfram@kriesing.de>
     *   @param      int     the id of the template
     *   @return     string  the complete path name to the template
     */
    function getFilename($id)
    {
        global $applError, $config;

        if (empty($id)) {
            return false;
        }

        $this->preset();
        if ($template = $this->get($id)) {
            // we hash the name, because there can be any kind of letter in the 'name'
            // which might produce an invalid filename ...
            $filename = $config->OOoTemplateDir . '/' . md5($template['name']
                      . $template['timestamp']) . '.' . $template['type'];
            if (@is_file($filename)) {
                return $filename;
            }
            if (!@is_dir($config->OOoTemplateDir)) {
                if (!@mkdir($config->OOoTemplateDir, 0775)) {
                    $applError->set('Could not write into the OpenOffice-Template directory!');
                    return false;
                }
            }

            /**
             * AK : just some words about OpenDocument-format :
             * An odt-file for example is in principle a zip-file with standardized content :
             * 
             * mimetype
             * Pictures\10000000000001E800000118B5A37F3F.png
             * layout-cache
             * content.xml
             * styles.xml
             * meta.xml
             * Thumbnails\thumbnail.png
             * settings.xml
             * meta-inf\manifest.xml
             * 
             */

            $templateData = $this->get($id, 'data');
            if ($templateData) {
                $fp = fopen($filename, 'w');
                if ($fp) {
                    fwrite($fp, $templateData);
                    fclose($fp);
                }
                if (@is_file($filename)) {
                    return $filename;
                }

                $applError->set('Error retreiving the template, please try again!');
                return false;
            }
        }
        return false;
    }

    /**
     *   this puts the file with the given id to the browser
     *   NOTE: this method puts the file out to the browser
     *   your script should die after this to not modify the data!
     *
     *   @version    2002/11/08
     *   @access     public
     *   @author     Wolfram Kriesing <wolfram@kriesing.de>
     *   @param      int     the id of the template
     *   @return     string  the complete path name to the template
     */
    function putFile($id)
    {
        global $config;

        $this->preset();
        $type = $this->get($id, 'type');

        require_once 'vp/Util/MimeType.php';
        header('Content-type: ' . vp_Util_MimeType::getMimeType($type));

        $filename = $this->getFilename($id);

        readfile($filename);
    }

} // end of class

$OOoTemplate = new modules_OOoTemplate;
